@@ -30,6 +30,7 @@
 #include "idorange.h"
 #include "idoscalemenuitem.h"
 #include "idotypebuiltins.h"
+#include "idoactionhelper.h"
 
 static void     ido_scale_menu_item_set_property           (GObject               *object,
                                                             guint                  prop_id,
@@ -51,9 +52,8 @@ static void     ido_scale_menu_item_primary_image_notify   (GtkImage
 static void     ido_scale_menu_item_secondary_image_notify (GtkImage              *image,
                                                             GParamSpec            *pspec,
                                                             IdoScaleMenuItem      *item);
-static void     ido_scale_menu_item_notify                 (IdoScaleMenuItem      *item,
-                                                            GParamSpec            *pspec,
-                                                            gpointer               user_data);
+static void     ido_scale_menu_item_parent_set             (GtkWidget             *item,
+                                                            GtkWidget             *previous_parent);
 static void     update_packing                             (IdoScaleMenuItem      *self,
                                                             IdoScaleMenuItemStyle  style);
 static void     default_primary_clicked_handler            (IdoScaleMenuItem      *self);
@@ -75,6 +75,7 @@ struct _IdoScaleMenuItemPrivate {
   IdoScaleMenuItemStyle style;
   IdoRangeStyle         range_style;
   gint                  toggle_size;
+  gboolean              ignore_value_changed;
 };
 
 enum {
@@ -82,6 +83,7 @@ enum {
   SLIDER_RELEASED,
   PRIMARY_CLICKED,
   SECONDARY_CLICKED,
+  VALUE_CHANGED,
   LAST_SIGNAL
 };
 
@@ -144,27 +146,15 @@ ido_scale_menu_item_size_allocate (GtkWidget     *widget,
   switch (priv->style)
     {
     case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
-#ifdef USE_GTK3
       gtk_widget_get_preferred_size (priv->primary_image, &primary_req, NULL);
       gtk_widget_get_preferred_size (priv->secondary_image, &secondary_req, NULL);
-#else
-      gtk_widget_get_child_requisition (priv->primary_image, &primary_req);
-      gtk_widget_get_child_requisition (priv->secondary_image, &secondary_req);
-#endif
-
       primary_padding = gtk_widget_get_visible (priv->primary_image) ? primary_req.width : 0;
       secondary_padding = gtk_widget_get_visible (priv->secondary_image) ? secondary_req.width : 0;
       break;
 
     case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
-#ifdef USE_GTK3
       gtk_widget_get_preferred_size (priv->primary_label, &primary_req, NULL);
       gtk_widget_get_preferred_size (priv->secondary_label, &secondary_req, NULL);
-#else
-      gtk_widget_get_child_requisition (priv->primary_label, &primary_req);
-      gtk_widget_get_child_requisition (priv->secondary_label, &secondary_req);
-#endif
-
       primary_padding = gtk_widget_get_visible (priv->primary_label) ? primary_req.width : 0;
       secondary_padding = gtk_widget_get_visible (priv->secondary_label) ? secondary_req.width : 0;
       break;
@@ -217,19 +207,18 @@ ido_scale_menu_item_toggle_size_allocate (IdoScaleMenuItem *item,
   priv->toggle_size = toggle_size;
 }
 
-static gboolean
-on_scale_button_press_or_release_event (GtkWidget      * widget G_GNUC_UNUSED, 
-                                        GdkEventButton * event,
-                                        gpointer         unused G_GNUC_UNUSED)
+static void
+ido_scale_menu_item_scale_value_changed (GtkRange *range,
+                                         gpointer  user_data)
 {
-  /* HACK: we want the behaviour you get with the middle button, so we
-   * mangle the event. clicking with other buttons moves the slider in
-   * step increments, clicking with the middle button moves the slider to
-   * the location of the click. */
-  if (event->button == 1)
-    event->button = 2;
+  IdoScaleMenuItem *self = user_data;
+  IdoScaleMenuItemPrivate *priv = GET_PRIVATE (self);
 
-  return FALSE;
+  /* The signal is not sent when it was set through
+   * ido_scale_menu_item_set_value().  */
+
+  if (!priv->ignore_value_changed)
+    g_signal_emit (self, signals[VALUE_CHANGED], 0, gtk_range_get_value (range));
 }
 
 static void
@@ -248,19 +237,11 @@ ido_scale_menu_item_constructed (GObject *object)
                 NULL);
 
   priv->scale = ido_range_new (adj, range_style);
+  g_signal_connect (priv->scale, "value-changed", G_CALLBACK (ido_scale_menu_item_scale_value_changed), self);
   g_object_ref (priv->scale);
   gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
-  g_signal_connect (G_OBJECT (priv->scale), "button-press-event",
-                    G_CALLBACK (on_scale_button_press_or_release_event), NULL);
-  g_signal_connect (G_OBJECT (priv->scale), "button-release-event",
-                    G_CALLBACK (on_scale_button_press_or_release_event), NULL);
 
-  
-#ifdef USE_GTK3
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#else
-  hbox = gtk_hbox_new (FALSE, 0);
-#endif
 
   priv->primary_image = gtk_image_new ();
   g_signal_connect (priv->primary_image, "notify",
@@ -283,10 +264,6 @@ ido_scale_menu_item_constructed (GObject *object)
                     G_CALLBACK (ido_scale_menu_item_toggle_size_allocate),
                     NULL);
 
-  g_signal_connect (self, "notify",
-                    G_CALLBACK (ido_scale_menu_item_notify),
-                    NULL);
-
   gtk_container_add (GTK_CONTAINER (self), hbox);
 
   gtk_widget_add_events (GTK_WIDGET(self), GDK_SCROLL_MASK);
@@ -306,6 +283,7 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
   widget_class->motion_notify_event  = ido_scale_menu_item_motion_notify_event;
   widget_class->scroll_event         = ido_scale_menu_item_scroll_event;
   widget_class->size_allocate        = ido_scale_menu_item_size_allocate;
+  widget_class->parent_set           = ido_scale_menu_item_parent_set;
 
   gobject_class->constructed  = ido_scale_menu_item_constructed;
   gobject_class->set_property = ido_scale_menu_item_set_property;
@@ -342,9 +320,15 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                    g_param_spec_boolean ("reverse-scroll-events",
                                                          "Reverse scroll events",
                                                          "Reverses how up/down scroll events are interpreted",
-                                                         FALSE,
+                                                         TRUE,
                                                          G_PARAM_READWRITE));
 
+  /**
+   * IdoScaleMenuItem::slider-grabbed:
+   * @menuitem: The #IdoScaleMenuItem emitting the signal.
+   *
+   * The ::slider-grabbed signal is emitted when the pointer selects the slider. 
+   */
   signals[SLIDER_GRABBED] = g_signal_new ("slider-grabbed",
                                           G_OBJECT_CLASS_TYPE (gobject_class),
                                           G_SIGNAL_RUN_FIRST,
@@ -353,6 +337,12 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                           g_cclosure_marshal_VOID__VOID,
                                           G_TYPE_NONE, 0);
 
+  /**
+   * IdoScaleMenuItem::slider-released:
+   * @menuitem: The #IdoScaleMenuItem emitting the signal.
+   *
+   * The ::slider-released signal is emitted when the pointer releases the slider.
+   */
   signals[SLIDER_RELEASED] = g_signal_new ("slider-released",
                                            G_OBJECT_CLASS_TYPE (gobject_class),
                                            G_SIGNAL_RUN_FIRST,
@@ -361,6 +351,12 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                            g_cclosure_marshal_VOID__VOID,
                                            G_TYPE_NONE, 0);
 
+  /**
+   * IdoScaleMenuItem::primary-clicked:
+   * @menuitem: The #IdoScaleMenuItem emitting the signal.
+   *
+   * The ::primary-clicked signal is emitted when the pointer clicks the primary label.
+   */
   signals[PRIMARY_CLICKED] = g_signal_new ("primary-clicked",
                                            G_TYPE_FROM_CLASS (item_class),
                                            G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
@@ -370,6 +366,12 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                            G_TYPE_NONE, /* return type */
                                            0 /* n_params */);
 
+  /**
+   * IdoScaleMenuItem::secondary-clicked:
+   * @menuitem: The #IdoScaleMenuItem emitting the signal.
+   *
+   * The ::secondary-clicked signal is emitted when the pointer clicks the secondary label.
+   */
   signals[SECONDARY_CLICKED] = g_signal_new ("secondary-clicked",
                                              G_TYPE_FROM_CLASS (item_class),
                                              G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
@@ -379,6 +381,23 @@ ido_scale_menu_item_class_init (IdoScaleMenuItemClass *item_class)
                                              G_TYPE_NONE, /* return type */
                                              0 /* n_params */);
 
+  /**
+   * IdoScaleMenuItem::value-changed:
+   * @menuitem: the #IdoScaleMenuItem for which the value changed
+   * @value: the new value
+   *
+   * Emitted whenever the value of the contained scale changes because
+   * of user input.
+   */
+  signals[VALUE_CHANGED] = g_signal_new ("value-changed",
+                                         IDO_TYPE_SCALE_MENU_ITEM,
+                                         G_SIGNAL_RUN_LAST,
+                                         0, NULL, NULL,
+                                         g_cclosure_marshal_VOID__DOUBLE,
+                                         G_TYPE_NONE,
+                                         1, G_TYPE_DOUBLE);
+
+
   g_type_class_add_private (item_class, sizeof (IdoScaleMenuItemPrivate));
 }
 
@@ -401,18 +420,18 @@ update_packing (IdoScaleMenuItem *self, IdoScaleMenuItemStyle style)
     {
     case IDO_SCALE_MENU_ITEM_STYLE_IMAGE:
       gtk_box_pack_start (box, priv->primary_image, FALSE, FALSE, 0);
-      gtk_box_pack_start (box, priv->scale, FALSE, FALSE, 0);
+      gtk_box_pack_start (box, priv->scale, TRUE, TRUE, 0);
       gtk_box_pack_start (box, priv->secondary_image, FALSE, FALSE, 0);
       break;
 
     case IDO_SCALE_MENU_ITEM_STYLE_LABEL:
       gtk_box_pack_start (box, priv->primary_label, FALSE, FALSE, 0);
-      gtk_box_pack_start (box, priv->scale, FALSE, FALSE, 0);
+      gtk_box_pack_start (box, priv->scale, TRUE, TRUE, 0);
       gtk_box_pack_start (box, priv->secondary_label, FALSE, FALSE, 0);
       break;
 
     default:
-      gtk_box_pack_start (box, priv->scale, FALSE, FALSE, 0);
+      gtk_box_pack_start (box, priv->scale, TRUE, TRUE, 0);
       break;
     }
 
@@ -422,6 +441,11 @@ update_packing (IdoScaleMenuItem *self, IdoScaleMenuItemStyle style)
 static void
 ido_scale_menu_item_init (IdoScaleMenuItem *self)
 {
+  IdoScaleMenuItemPrivate *priv = GET_PRIVATE (self);
+
+  priv->reverse_scroll = TRUE;
+
+  gtk_widget_set_size_request (GTK_WIDGET (self), 200, -1);
 }
 
 static void
@@ -505,10 +529,6 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
   IdoScaleMenuItemPrivate *priv = GET_PRIVATE (menuitem);
   gdouble x;
   
-#ifndef USE_GTK3
-  GtkWidget *scale = priv->scale;
-#endif
-
   // can we block emissions of "grab-notify" on parent??
 
   translate_event_coordinates (menuitem, event->x, &x);
@@ -517,28 +537,16 @@ ido_scale_menu_item_button_press_event (GtkWidget      *menuitem,
   translate_event_coordinates (menuitem, event->x_root, &x);
   event->x_root = x;
 
-#ifndef USE_GTK3
-  ubuntu_gtk_widget_set_has_grab (scale, TRUE);
-#endif
-
   gtk_widget_event (priv->scale,
                     ((GdkEvent *)(void*)(event)));
 
-#ifndef USE_GTK3
-  ubuntu_gtk_widget_set_has_grab (scale, FALSE);
-#endif
-
   if (!priv->grabbed)
     {
       priv->grabbed = TRUE;
       g_signal_emit (menuitem, signals[SLIDER_GRABBED], 0);
     }
 
-#ifdef USE_GTK3
   return FALSE;
-#else
-  return TRUE;
-#endif
 }
 
 static gboolean
@@ -633,21 +641,18 @@ menu_hidden (GtkWidget        *menu,
 }
 
 static void
-ido_scale_menu_item_notify (IdoScaleMenuItem *item,
-                            GParamSpec       *pspec,
-                            gpointer          user_data)
-{
-  if (g_strcmp0 (pspec->name, "parent"))
-    {
-      GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (item));
+ido_scale_menu_item_parent_set (GtkWidget *item,
+                                GtkWidget *previous_parent)
 
-      if (parent)
-        {
-          g_signal_connect (parent, "hide",
-                            G_CALLBACK (menu_hidden),
-                            item);
-        }
-    }
+{
+  GtkWidget *parent;
+
+  if (previous_parent)
+    g_signal_handlers_disconnect_by_func (previous_parent, menu_hidden, item);
+
+  parent = gtk_widget_get_parent (item);
+  if (parent)
+    g_signal_connect (parent, "hide", G_CALLBACK (menu_hidden), item);
 }
 
 static void
@@ -677,9 +682,10 @@ ido_scale_menu_item_secondary_image_notify (GtkImage         *image,
  * @label: the text of the new menu item.
  * @size: The size style of the range.
  * @adjustment: A #GtkAdjustment describing the slider value.
- * @returns: a new #IdoScaleMenuItem.
  *
  * Creates a new #IdoScaleMenuItem with an empty label.
+ *
+ * Return Value: a new #IdoScaleMenuItem.
  **/
 GtkWidget*
 ido_scale_menu_item_new (const gchar   *label,
@@ -699,9 +705,10 @@ ido_scale_menu_item_new (const gchar   *label,
  * @min: The minimum value of the slider.
  * @max: The maximum value of the slider.
  * @step: The step increment of the slider.
- * @returns: a new #IdoScaleMenuItem.
  *
  * Creates a new #IdoScaleMenuItem containing a label.
+ *
+ * Return Value: a new #IdoScaleMenuItem.
  **/
 GtkWidget*
 ido_scale_menu_item_new_with_range (const gchar  *label,
@@ -723,9 +730,10 @@ ido_scale_menu_item_new_with_range (const gchar  *label,
 /**
  * ido_scale_menu_item_get_scale:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A pointer to the scale widget.
  *
  * Retrieves the scale widget.
+ *
+ * Return Value: (transfer none): The #IdoRange in this item
  **/
 GtkWidget*
 ido_scale_menu_item_get_scale (IdoScaleMenuItem *menuitem)
@@ -742,10 +750,11 @@ ido_scale_menu_item_get_scale (IdoScaleMenuItem *menuitem)
 /**
  * ido_scale_menu_item_get_style:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A #IdoScaleMenuItemStyle enum describing the style.
  *
  * Retrieves the type of widgets being used for the primary and
  * secondary widget slots.  This could be images, labels, or nothing.
+ *
+ * Return Value: A #IdoScaleMenuItemStyle enum describing the style.
  **/
 IdoScaleMenuItemStyle
 ido_scale_menu_item_get_style (IdoScaleMenuItem *menuitem)
@@ -759,6 +768,14 @@ ido_scale_menu_item_get_style (IdoScaleMenuItem *menuitem)
   return priv->style;
 }
 
+/**
+ * ido_scale_menu_item_set_style:
+ * @menuitem: The #IdoScaleMenuItem
+ * @style: Set the style use for the primary and secondary widget slots.
+ *
+ * Sets the type of widgets being used for the primary and
+ * secondary widget slots.  This could be images, labels, or nothing.
+ **/
 void
 ido_scale_menu_item_set_style (IdoScaleMenuItem      *menuitem,
                                IdoScaleMenuItemStyle  style)
@@ -777,11 +794,12 @@ ido_scale_menu_item_set_style (IdoScaleMenuItem      *menuitem,
 /**
  * ido_scale_menu_item_get_primary_image:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A #GtkWidget pointer for the primary image.
  *
  * Retrieves a pointer to the image widget used in the primary slot.
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
+ *
+ * Return Value: (transfer none): A #GtkWidget pointer for the primary image.
  **/
 GtkWidget *
 ido_scale_menu_item_get_primary_image (IdoScaleMenuItem *menuitem)
@@ -798,11 +816,12 @@ ido_scale_menu_item_get_primary_image (IdoScaleMenuItem *menuitem)
 /**
  * ido_scale_menu_item_get_secondary_image:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A #GtkWidget pointer for the secondary image.
  *
  * Retrieves a pointer to the image widget used in the secondary slot.
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
+ *
+ * Return Value: (transfer none): A #GtkWidget pointer for the secondary image.
  **/
 GtkWidget *
 ido_scale_menu_item_get_secondary_image (IdoScaleMenuItem *menuitem)
@@ -816,14 +835,46 @@ ido_scale_menu_item_get_secondary_image (IdoScaleMenuItem *menuitem)
   return priv->secondary_image;
 }
 
+/**
+ * ido_scale_menu_item_set_icons:
+ * @item: a #IdoScaleMenuItem
+ * @primary-icon: (allow-none): the primary icon, or %NULL
+ * @secondary-icon: (allow-none): the secondary icon, %NULL
+ *
+ * Sets the icons of @item to  @primary_icon and @secondary_icon.
+ * Pass %NULL for either of them to unset the icon.
+ */
+static void
+ido_scale_menu_item_set_icons (IdoScaleMenuItem *item,
+                               GIcon            *primary_icon,
+                               GIcon            *secondary_icon)
+{
+  GtkWidget *primary;
+  GtkWidget *secondary;
+
+  primary = ido_scale_menu_item_get_primary_image (item);
+  secondary = ido_scale_menu_item_get_secondary_image (item);
+
+  if (primary_icon)
+    gtk_image_set_from_gicon (GTK_IMAGE (primary), primary_icon, GTK_ICON_SIZE_MENU);
+  else
+    gtk_image_clear (GTK_IMAGE (primary));
+
+  if (secondary_icon)
+    gtk_image_set_from_gicon (GTK_IMAGE (secondary), secondary_icon, GTK_ICON_SIZE_MENU);
+  else
+    gtk_image_clear (GTK_IMAGE (secondary));
+}
+
 /**
  * ido_scale_menu_item_get_primary_label:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A const gchar* string of the label text.
  *
  * Retrieves a string of the text for the primary label widget.
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
+ *
+ * Return Value: The label text.
  **/
 const gchar*
 ido_scale_menu_item_get_primary_label (IdoScaleMenuItem *menuitem)
@@ -838,13 +889,14 @@ ido_scale_menu_item_get_primary_label (IdoScaleMenuItem *menuitem)
 }
 
 /**
- * ido_scale_menu_item_get_primary_label:
+ * ido_scale_menu_item_get_secondary_label:
  * @menuitem: The #IdoScaleMenuItem
- * @returns: A const gchar* string of the label text.
  *
- * Retrieves a string of the text for the primary label widget.
+ * Retrieves a string of the text for the secondary label widget.
  * Whether this is visible depends upon the return value from
  * ido_scale_menu_item_get_style().
+ *
+ * Return Value: The label text.
  **/
 const gchar*
 ido_scale_menu_item_get_secondary_label (IdoScaleMenuItem *menuitem)
@@ -861,7 +913,7 @@ ido_scale_menu_item_get_secondary_label (IdoScaleMenuItem *menuitem)
 /**
  * ido_scale_menu_item_set_primary_label:
  * @menuitem: The #IdoScaleMenuItem
- * @label: A string containing the label text
+ * @label: The label text
  *
  * Sets the text for the label widget in the primary slot.  This
  * widget will only be visibile if the return value of
@@ -884,11 +936,11 @@ ido_scale_menu_item_set_primary_label (IdoScaleMenuItem *menuitem,
 }
 
 /**
- * ido_scale_menu_item_set_primary_label:
+ * ido_scale_menu_item_set_secondary_label:
  * @menuitem: The #IdoScaleMenuItem
- * @label: A string containing the label text
+ * @label: The label text
  *
- * Sets the text for the label widget in the primary slot.  This
+ * Sets the text for the label widget in the secondary slot.  This
  * widget will only be visibile if the return value of
  * ido_scale_menu_item_get_style() is set to %IDO_SCALE_MENU_ITEM_STYLE_LABEL.
  **/
@@ -911,16 +963,16 @@ ido_scale_menu_item_set_secondary_label (IdoScaleMenuItem *menuitem,
 /**
  * ido_scale_menu_item_primary_clicked:
  * @menuitem: the #IdoScaleMenuItem
- * 
+ *
  * Emits the "primary-clicked" signal.
  *
  * The default handler for this signal lowers the scale's
  * adjustment to its lower bound.
  */
 void
-ido_scale_menu_item_primary_clicked (IdoScaleMenuItem * item)
+ido_scale_menu_item_primary_clicked (IdoScaleMenuItem * menuitem)
 {
-  g_signal_emit (item, signals[PRIMARY_CLICKED], 0);
+  g_signal_emit (menuitem, signals[PRIMARY_CLICKED], 0);
 }
 static void
 default_primary_clicked_handler (IdoScaleMenuItem * item)
@@ -932,18 +984,18 @@ default_primary_clicked_handler (IdoScaleMenuItem * item)
 }
 
 /**
- * ido_scale_menu_item_primary_clicked:
+ * ido_scale_menu_item_secondary_clicked:
  * @menuitem: the #IdoScaleMenuItem
- * 
- * Emits the "primary-clicked" signal.
+ *
+ * Emits the "secondary-clicked" signal.
  *
  * The default handler for this signal raises the scale's
  * adjustment to its upper bound.
  */
 void
-ido_scale_menu_item_secondary_clicked (IdoScaleMenuItem * item)
+ido_scale_menu_item_secondary_clicked (IdoScaleMenuItem * menuitem)
 {
-  g_signal_emit (item, signals[SECONDARY_CLICKED], 0);
+  g_signal_emit (menuitem, signals[SECONDARY_CLICKED], 0);
 }
 static void
 default_secondary_clicked_handler (IdoScaleMenuItem * item)
@@ -954,4 +1006,115 @@ default_secondary_clicked_handler (IdoScaleMenuItem * item)
   gtk_adjustment_set_value (adj, gtk_adjustment_get_upper (adj));
 }
 
+/* ido_scale_menu_item_set_value:
+ *
+ * Sets the value of the scale inside @item to @value, without emitting
+ * "value-changed".
+ */
+static void
+ido_scale_menu_item_set_value (IdoScaleMenuItem *item,
+                               gdouble           value)
+{
+  IdoScaleMenuItemPrivate *priv = GET_PRIVATE (item);
+
+  /* set ignore_value_changed to signify to the scale menu item that it
+   * should not emit its own value-changed signal, as that should only
+   * be emitted when the value is changed by the user. */
+
+  priv->ignore_value_changed = TRUE;
+  gtk_range_set_value (GTK_RANGE (priv->scale), value);
+  priv->ignore_value_changed = FALSE;
+}
+
+/**
+ * ido_scale_menu_item_state_changed:
+ *
+ * Updates a IdoScaleMenuItem from @state. State must be a double which
+ * contains the current value of the slider.
+ */
+static void
+ido_scale_menu_item_state_changed (IdoActionHelper *helper,
+                                   GVariant        *state,
+                                   gpointer         user_data)
+{
+  GtkWidget *menuitem;
+
+  menuitem = ido_action_helper_get_widget (helper);
+  ido_scale_menu_item_set_value (IDO_SCALE_MENU_ITEM (menuitem), g_variant_get_double (state));
+}
+
+static void
+ido_scale_menu_item_value_changed (GtkScale *scale,
+                                   gdouble   value,
+                                   gpointer  user_data)
+{
+  IdoActionHelper *helper = user_data;
+
+  ido_action_helper_change_action_state (helper, g_variant_new_double (value));
+}
+
+static GIcon *
+menu_item_get_icon (GMenuItem   *menuitem,
+                    const gchar *attribute)
+{
+  GVariant *value;
+
+  value = g_menu_item_get_attribute_value (menuitem, attribute, NULL);
+
+  return value ? g_icon_deserialize (value) : NULL;
+}
+
+/**
+ * ido_scale_menu_item_new_from_model:
+ *
+ * Creates a new #IdoScaleMenuItem. If @menuitem contains an action, it
+ * will be bound to that action in @actions.
+ *
+ * Returns: (transfer full): a new #IdoScaleMenuItem
+ */
+GtkMenuItem *
+ido_scale_menu_item_new_from_model (GMenuItem    *menuitem,
+                                    GActionGroup *actions)
+{
+  GtkWidget *item;
+  gchar *action;
+  gdouble min = 0.0;
+  gdouble max = 100.0;
+  gdouble step = 1.0;
+  GIcon *min_icon;
+  GIcon *max_icon;
+
+  g_menu_item_get_attribute (menuitem, "min-value", "d", &min);
+  g_menu_item_get_attribute (menuitem, "max-value", "d", &max);
+  g_menu_item_get_attribute (menuitem, "step", "d", &step);
+
+  item = ido_scale_menu_item_new_with_range ("Volume", IDO_RANGE_STYLE_DEFAULT, 0.0, min, max, step);
+  ido_scale_menu_item_set_style (IDO_SCALE_MENU_ITEM (item), IDO_SCALE_MENU_ITEM_STYLE_IMAGE);
+
+  if (g_menu_item_get_attribute (menuitem, "action", "s", &action))
+    {
+      IdoActionHelper *helper;
+
+      helper = ido_action_helper_new (item, actions, action, NULL);
+      g_signal_connect (helper, "action-state-changed",
+                        G_CALLBACK (ido_scale_menu_item_state_changed), NULL);
+
+      g_signal_connect (item, "value-changed", G_CALLBACK (ido_scale_menu_item_value_changed), helper);
+      g_signal_connect_swapped (item, "destroy", G_CALLBACK (g_object_unref), helper);
+
+      g_free (action);
+    }
+
+  min_icon = menu_item_get_icon (menuitem, "min-icon");
+  max_icon = menu_item_get_icon (menuitem, "max-icon");
+  ido_scale_menu_item_set_icons (IDO_SCALE_MENU_ITEM (item), min_icon, max_icon);
+
+  if (min_icon)
+    g_object_unref (min_icon);
+  if (max_icon)
+  g_object_unref (max_icon);
+
+  return GTK_MENU_ITEM (item);
+}
+
 #define __IDO_SCALE_MENU_ITEM_C__
